<?php
/**
 * Created by PhpStorm.
 * User: Milad Rahimi <info@miladrahimi.com>
 * Date: 6/23/2018 AD
 * Time: 01:57
 */

namespace MiladRahimi\PhpRouter\Tests;

use MiladRahimi\PhpRouter\Enums\HttpMethods;
use MiladRahimi\PhpRouter\Router;
use MiladRahimi\PhpRouter\Services\PublisherInterface;
use MiladRahimi\PhpRouter\Tests\Classes\Publisher;
use MiladRahimi\PhpRouter\Tests\Classes\SampleController;
use PHPUnit\Framework\TestCase as BaseTestCase;

class TestCase extends BaseTestCase
{
    protected function setUp()
    {
        parent::setUp();

        $this->mockRequest(HttpMethods::GET, 'http://example.com/');
    }

    /**
     * Mock http server request ($_SERVER)
     *
     * @param string $method
     * @param string $url
     */
    protected function mockRequest(string $method, string $url)
    {
        $urlParts = parse_url($url);

        $_SERVER['SERVER_NAME'] = $urlParts['scheme'] . '://' . $urlParts['host'];
        $_SERVER['REQUEST_URI'] = ($urlParts['path'] ?? '/') . '?' . ($urlParts['query'] ?? '');
        $_SERVER['REQUEST_METHOD'] = $method;
    }

    /**
     * Get router instance with mocked properties
     *
     * @return Router
     */
    protected function createRouter(): Router
    {
        $router = new Router();
        $router->setPublisher(new Publisher());

        return $router;
    }

    /**
     * Get the simplest controller
     *
     * @return string
     */
    protected function simpleController(): string
    {
        return SampleController::class . '@getNoParameter';
    }

    /**
     * Get output generated by current request after the router dispatch
     *
     * @param Router $router
     * @return string
     */
    protected function getOutput(Router $router)
    {
        /** @var Publisher $publisher */
        $publisher = $router->getPublisher();

        return $publisher->output;
    }

    /**
     * Get router publisher
     *
     * @param Router $router
     * @return PublisherInterface|Publisher
     */
    protected function getPublisher(Router $router): Publisher
    {
        return $router->getPublisher();
    }
}
